//**********************
// OBD-II
//
// Condition:
// Header: Config.h, LCD.h, Wait.h
// Source: LCD.c, Wait.c
// Library: libp30F4012-coff.a
// Linker: p30f4012.gld
//
// (C)Copyright 2009 All rights reserved by Y.Onodera
// http://einst.hp.infoseek.co.jp
//**********************
#include <p30f4012.h>
#include <stdio.h>
#include "Config.h"
#include "LCD.h"
#include "RS232C.h"
#include "Wait.h"
#include "SW.h"
#include "K.h"
#include "CAN1.h"


int buffers;
unsigned char buf[12];
int flag;
int mode;
int set;
unsigned long PIDstatus00;
unsigned long PIDstatus20;
unsigned long PIDstatus40;


void InitPort(){

	K_TX_TRIS=0;	// set output
	K_TX_PORT=1;
	K_RX_TRIS=1;	// set input
	CAN_TX_TRIS=0;	// set output
	CAN_TX_PORT=1;
	CAN_RX_TRIS=0;	// set output
	CAN_RX_TRIS=1;	// set input
	K_error=0;
	CAN_error=0;

}


// Main routine
int main(void)
{

	unsigned char j;
	int init;
	buffers=0;
	flag=0;
	mode=0;
	set=0;
	init=0;

	InitPort();
	InitSW();
	// enable interrupt
	DISICNT = 0x0000;

	// LCD init
	InitLCD();
	// turn on
	PutCmdLCD(DISPLAY_ON & CURSOR_OFF & BLINK_OFF);
	ClsLCD();

	PIDstatus00=0;
	PIDstatus20=0;
	PIDstatus40=0;

	while(1){

		while(set==0){
			LocateLCD(0,0);
			switch(mode){
			case 0:
				PutsLCD("K   scan");
				break;
			case 1:
				PutsLCD("CAN scan");
				break;
			case 2:
				PutsLCD("K   DTC ");
				break;
			case 3:
				PutsLCD("CAN DTC ");
				break;
			case 4:
				PutsLCD("K   clr ");
				break;
			case 5:
				PutsLCD("CAN clr ");
				break;
			case 6:
				PutsLCD("K   test");
				break;
			case 7:
				PutsLCD("CAN test");
				break;
			}
			LocateLCD(0,1);
			if(j==0){
				PutsLCD("J1=Mode ");
				j=1;
			}else{
				PutsLCD("J2=Start");
				j=0;
			}
			WaitNs(1);
			flag=1;
			init=0;
		}


		while(set==1){

			if(init==0){
				switch(mode){
				case 0:
				case 2:
				case 4:
					InitK();
					break;
				case 6:
					InitKt();
					break;
				case 1:
				case 3:
				case 5:
				case 7:
					InitCAN1();
					break;
				}
				init=1;
			}


			// K scan
			if(mode==0){
				Request9141(0);
				Recieve9141(0);
				Print9141();
				for(j=1;j<=0x20;j++){
					if(PIDstatus00 >> (0x20-j) & 0x01){
						Request9141(j);
						Recieve9141(j);
						Print9141();
					}
					if(set==0){	// to exit
						break;
					}
				}
				for(j=0x21;j<=0x40;j++){
					if(PIDstatus20 >> (0x40-j) & 0x01){
						Request9141(j);
						Recieve9141(j);
						Print9141();
					}
					if(set==0){	// to exit
						break;
					}
				}
				for(j=0x41;j<0x60;j++){
					if(PIDstatus40 >> (0x60-j) & 0x01){
						Request9141(j);
						Recieve9141(j);
						Print9141();
					}
					if(set==0){	// to exit
						break;
					}
				}
			}

			// CAN scan
			if(mode==1){
				RequestCAN(0);
				RecieveCAN();
				PrintCAN();
				for(j=1;j<=0x20;j++){
					if(PIDstatus00 >> (0x20-j) & 0x01){
						RequestCAN(j);
						RecieveCAN();
						PrintCAN();
					}
					if(set==0){	// to exit
						break;
					}
				}
				for(j=0x21;j<=0x40;j++){
					if(PIDstatus20 >> (0x40-j) & 0x01){
						RequestCAN(j);
						RecieveCAN();
						PrintCAN();
					}
					if(set==0){	// to exit
						break;
					}
				}
				for(j=0x41;j<0x60;j++){
					if(PIDstatus40 >> (0x60-j) & 0x01){
						RequestCAN(j);
						RecieveCAN();
						PrintCAN();
					}
					if(set==0){	// to exit
						break;
					}
				}
			}

			// K DTC
			if(mode==2){
				Request9141DTC();
				Recieve9141DTC();
				for(j=1;j<=3;j++){
					Print9141DTC(j);
				}
			}

			// CAN DTC
			if(mode==3){
				RequestDTC();
				RecieveCAN();
				for(j=1;j<=3;j++){
					PrintDTC(j);
				}
			}

			// K clear
			if(mode==4){
				Request9141CLR();
				set=0;
			}

			// CAN clear
			if(mode==5){
				RequestCLR();
				set=0;
			}

			// K test
			if(mode==6){
				Recieve9141(0x4f);
				SendK();
			}

			// CAN test
			if(mode==7){
				RecieveCAN();
				SendCAN();
			}

		}

	CloseCOM1();
	}

}

//char tmp[20];
//LocateLCD(0,1);
//sprintf(tmp,"%02X%02X%02X%02X",buf[0],buf[1],buf[2],buf[3]);
//PutsLCD(tmp);
//WaitNs(3);

